/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2005 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.VariableOperand;

import java.util.Arrays;
import java.util.regex.Pattern;

/**
 * Is an operator representing the LIKE or the NOT LIKE operator. The regular 
 * expressions are used for the string comparaison. The current memory
 * evaluation does not support escape characters.
 *  
 * @author S.Chassande-Barrioz
 */
public class Like extends BasicOperator implements Operator {

    private static final long serialVersionUID = 7364254448772319728L;
    
    /**
     * Indicates if this operator is a Like or a NOT LIKE. 
     */
    boolean not = false;
    
    /**
     * Pattern matcher for the memory evaluation  
     */
    private Pattern pattern = null;

	public Like() {
	}

	public Like(Expression str, Expression pattern, Expression escape) {
		super(new Expression[] {str, pattern, escape});
	}

	public Like(Expression str, Expression pattern, Expression escape, boolean not) {
		super(new Expression[] {str, pattern, escape});
		this.not = not;
	}

	public Like(Expression str, Expression pattern) {
		super(new Expression[] {str, pattern});
	}

	public Like(Expression str, Expression pattern, boolean not) {
		super(new Expression[] {str, pattern});
		this.not = not;
	}

	public Object clone(Object clone, java.util.Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
        ((Like) clone).not = not;
        ((Like) clone).pattern = pattern;
		return clone;
	}

	// IMPLEMENTATION OF THE Operator INTERFACE //
	//------------------------------------------//

	public String getOperatorString() {
		return (not ? "NOT LIKE" : "LIKE");
	}
    
    /**
     * compiles the operator:
     * - compiles operands
     * - verify type of parameter
     * - compute the result holder (operand)
     * - try to compile the pattern matcher if the filter is already defined
     */
	public Operand compileExpression()
		throws ExpressionException, MalformedExpressionException {
        compileOperands();
        if (!verified) {
            checkOperands(new PType[][]{
                    {PTypeSpace.STRING, PTypeSpace.STRING},
                    {PTypeSpace.STRING, PTypeSpace.STRING, PTypeSpace.STRING},
            });
            verified = true;
        }
        if (result != null) {
            result = new BasicVariableOperand(PTypeSpace.BOOLEAN);
        }
        if (expressions[1] instanceof Operand 
                && !(expressions[1] instanceof VariableOperand)) {
            // the pattern is a operand which will not change, 
            // so we can compile a pattern matcher immediatly
            try {
                pattern = Pattern.compile(((Operand) expressions[1]).getString());
            } catch (TypingException e) {
                throw new ExpressionException(e);
            }
        }
        return result;
	}

    /**
     * Evaluates the operator on operands fetch from the sub expressions.
     */
	public Operand evaluate(ParameterOperand[] values, Object o)
		throws ExpressionException {
        if (pattern == null) {
            try {
                pattern = Pattern.compile(
                        expressions[1].evaluate(values, o).getString());
            } catch (TypingException e) {
                throw new ExpressionException(e);
            }
        }
        boolean eval = pattern.matcher(
                expressions[0].evaluate(values, o).getString()).matches();
        result.setValue(not ^ eval);
        return result;
	}
}
